// Testing uSD read and write using FSRW
// Note:  FSRW is limited to 8.3 file format and only the root directory

#include <stdio.h>
#include <propeller.h>

// to get the frequency we desire, we
// specify P2_TARGET_MHZ then include "sys/p2es_clock.h"
// e.g. to get 160_000_000, define P2_TARGET_MHZ to 160
// p2es_clock.h defines constants _SETFREQ and _CLOCKFREQ
// which we can then use to set the clock
//
// note that this is not the only way to set the clock; you can
// certainly calculate the desired clock mode based on the frequency
// and pass the resulting mode and frequency directly to clkset()
// p2es_clock.h is just a convenience header
#define P2_TARGET_MHZ 160
#include "sys/p2es_clock.h"

#ifndef _BAUD
#define _BAUD 230400
#endif


// the Spin2 uSD driver
struct __using("fsrw.spin2") sd;


//Which type of board are we using?
#define Board_Eval  //Parallax Evaluation Board
//#define Board_Board1  //RJA's first working P2 board

#ifdef Board_Eval
#define sd_miso 58
#define sd_mosi 59
#define sd_cs 60
#define sd_clk 61
#endif // Board_Eval

#ifdef Board_Board1
#define sd_miso 44
#define sd_mosi 46
#define sd_cs 47
#define sd_clk 45
#endif // Board_Board1

// and the main program
void main() 
{
    printf("FSRW uSD interface test program\n");
    sleep(1);

    printf("Trying to mount...\n");
    int i = sd.mount_explicit(sd_miso, sd_clk, sd_mosi, sd_cs);
    printf("Mount returns: %d   (1=SD, 2=SDHC, negative=error)\n", i);
    if (i < 0)
    {
        printf("Mount failed.  Check board definition and presense of working uSD card.\n");
        for (;;) {}
    }

    printf("Attempting to open a file (fsrwtest.txt) for writing...\n");
    char filename[] = "fsrwtest.txt";
    i = sd.popen("fsrwtest.txt", 'w');
    if (i < 0)
    {
        printf("Open for write failed, returned %d.  Check write protect switch.\n",i);
        for (;;) {}
    }

    printf("Writing to test file...\n");
    char teststring[] = "It works!\r\n";
    i = sd.pwrite(teststring, sizeof(teststring));
    if (i != sizeof(teststring))
    {
        printf("Write failed.  Should have returned %d, but instead returned %d", sizeof(teststring),i);
        for (;;) {}
    }
    sd.pclose();

    printf("Re-Opening and then Reading from test file...\n");
    i = sd.popen("fsrwtest.txt", 'r');
    if (i < 0)
    {
        printf("Open for read failed.\n");
        for (;;) {}
    }


    int s = sd.fsize();
    printf("File size = %d bytes\n",s);

    char inbuffer[256];
    i = sd.pread(inbuffer,256);
    if (i != s)
    {
        printf("Read failed.  Should have returned %d, but instead returned %d\n", s, i);
        for (;;) {}
    }
    sd.pclose();

    printf("File contents: \n");
    printf(inbuffer);

    printf("\nAll done!");
    for(;;) {}

}
